@@ -32,9 +32,9 @@
 #include <string>
 #include <vector>
 
-#include "acc_user.h"
-#include "acc_vhdl.h"  // Messy :(
-#include "mti.h"
+#include "_vendor/fli/acc_user.h"
+#include "_vendor/fli/acc_vhdl.h"  // Messy :(
+#include "_vendor/fli/mti.h"
 
 extern "C" {
 static FliProcessCbHdl *sim_init_cb;
@@ -244,8 +244,7 @@ GpiObjHdl *FliImpl::create_gpi_obj_from_handle(void *hdl, std::string &name,
     return new_obj;
 }
 
-GpiObjHdl *FliImpl::native_check_create(void *raw_hdl, GpiObjHdl *parent) {
-    COCOTB_UNUSED(parent);
+GpiObjHdl *FliImpl::native_check_create(void *raw_hdl, GpiObjHdl *) {
     LOG_DEBUG("Trying to convert a raw handle to an FLI Handle.");
 
     const char *c_name = acc_fetch_name(raw_hdl);
@@ -335,6 +334,12 @@ GpiObjHdl *FliImpl::native_check_create(std::string &name, GpiObjHdl *parent) {
     } else if (search_rgn) {
         mtiRegionIdT rgn;
 
+        // Looking for generates should only occur if the parent is from this
+        // implementation
+        if (!parent->is_this_impl(fli_table)) {
+            return NULL;
+        }
+
         /* If not found, check to see if the name of a generate loop and create
          * a pseudo-region */
         for (rgn = mti_FirstLowerRegion(parent->get_handle<mtiRegionIdT>());
@@ -456,8 +461,7 @@ GpiObjHdl *FliImpl::native_check_create(int32_t index, GpiObjHdl *parent) {
     }
 }
 
-const char *FliImpl::reason_to_string(int reason) {
-    COCOTB_UNUSED(reason);
+const char *FliImpl::reason_to_string(int) {
     return "Who can explain it, who can tell you why?";
 }
 
@@ -559,6 +563,7 @@ error:
 }
 
 GpiCbHdl *FliImpl::register_timed_callback(uint64_t time) {
+    // get time from cache instead of allocating
     FliTimedCbHdl *hdl = cache.get_timer(time);
 
     if (hdl->arm_callback()) {
@@ -1053,12 +1058,26 @@ FliTimedCbHdl *FliTimerCache::get_timer(uint64_t time) {
     return hdl;
 }
 
-void FliTimerCache::put_timer(FliTimedCbHdl *hdl) { free_list.push(hdl); }
+static constexpr size_t FLI_TIMER_CACHE_SIZE =
+    256;  // Arbitrary large value, it's doubtful more than 256 simultaneous
+          // Timer triggers will be active at any time
+
+void FliTimerCache::put_timer(FliTimedCbHdl *hdl) {
+    // save FLI_TIMER_CACHE_SIZE Timer objects before deleting, this should
+    // prevent "live leaking"
+    if (free_list.size() < FLI_TIMER_CACHE_SIZE) {
+        free_list.push(hdl);
+    } else {
+        delete hdl;
+    }
+}
 
 extern "C" {
 
 // Main re-entry point for callbacks from simulator
 void handle_fli_callback(void *data) {
+    gpi_to_user();
+
     fflush(stderr);
 
     FliProcessCbHdl *cb_hdl = (FliProcessCbHdl *)data;
@@ -1078,12 +1097,19 @@ void handle_fli_callback(void *data) {
         gpi_cb_state_e new_state = cb_hdl->get_call_state();
 
         /* We have re-primed in the handler */
-        if (new_state != GPI_PRIMED)
-            if (cb_hdl->cleanup_callback()) delete cb_hdl;
+        if (new_state != GPI_PRIMED) {
+            if (cb_hdl->cleanup_callback()) {
+                delete cb_hdl;
+            }
+        }
     } else {
         /* Issue #188 seems to appear via FLI as well */
-        cb_hdl->cleanup_callback();
+        if (cb_hdl->cleanup_callback()) {
+            delete cb_hdl;
+        }
     }
+
+    gpi_to_simulator();
 };
 
 static void register_initial_callback() {
@@ -1096,19 +1122,19 @@ static void register_final_callback() {
     sim_finish_cb->arm_callback();
 }
 
-static void register_embed() {
+static void register_impl() {
     fli_table = new FliImpl("FLI");
     gpi_register_impl(fli_table);
 }
 
 void cocotb_init() {
     LOG_INFO("cocotb_init called");
-    register_embed();
-    gpi_load_extra_libs();
+    register_impl();
+    gpi_entry_point();
     register_initial_callback();
     register_final_callback();
 }
 
 }  // extern "C"
 
-GPI_ENTRY_POINT(cocotbfli, register_embed);
+GPI_ENTRY_POINT(cocotbfli, register_impl);
