@@ -92,7 +92,9 @@ static GpiHandleStore unique_handles;
 
 #endif
 
-size_t gpi_print_registered_impl() {
+static bool sim_ending = false;
+
+static size_t gpi_print_registered_impl() {
     vector<GpiImplInterface *>::iterator iter;
     for (iter = registered_impls.begin(); iter != registered_impls.end();
          iter++) {
@@ -123,20 +125,19 @@ void gpi_embed_init(int argc, char const *const *argv) {
 
 void gpi_embed_end() {
     embed_sim_event(SIM_FAIL, "Simulator shutdown prematurely");
-    gpi_cleanup();
+    sim_ending = true;
 }
 
-void gpi_sim_end() { registered_impls[0]->sim_end(); }
+void gpi_sim_end() {
+    registered_impls[0]->sim_end();
+    sim_ending = true;
+}
 
 void gpi_cleanup(void) {
     CLEAR_STORE();
     embed_sim_cleanup();
 }
 
-void gpi_embed_event(gpi_event_t level, const char *msg) {
-    embed_sim_event(level, msg);
-}
-
 static void gpi_load_libs(std::vector<std::string> to_load) {
     std::vector<std::string>::iterator iter;
 
@@ -178,7 +179,7 @@ static void gpi_load_libs(std::vector<std::string> to_load) {
     }
 }
 
-void gpi_load_extra_libs() {
+void gpi_entry_point() {
     /* Lets look at what other libs we were asked to load too */
     char *lib_env = getenv("GPI_EXTRA");
 
@@ -256,21 +257,34 @@ gpi_sim_hdl gpi_get_root_handle(const char *name) {
     }
 }
 
-static GpiObjHdl *__gpi_get_handle_by_name(GpiObjHdl *parent, std::string name,
-                                           GpiImplInterface *skip_impl) {
-    vector<GpiImplInterface *>::iterator iter;
-
-    GpiObjHdl *hdl = NULL;
-
+static GpiObjHdl *gpi_get_handle_by_name_(GpiObjHdl *parent, std::string name,
+                                          GpiImplInterface *skip_impl) {
     LOG_DEBUG("Searching for %s", name.c_str());
 
-    for (iter = registered_impls.begin(); iter != registered_impls.end();
+    // check parent impl *first* if it's not skipped
+    if (!skip_impl || (skip_impl != parent->m_impl)) {
+        auto hdl = parent->m_impl->native_check_create(name, parent);
+        if (hdl) {
+            return CHECK_AND_STORE(hdl);
+        }
+    }
+
+    // iterate over all registered impls to see if we can get the signal
+    for (auto iter = registered_impls.begin(); iter != registered_impls.end();
          iter++) {
+        // check if impl is skipped
         if (skip_impl && (skip_impl == (*iter))) {
             LOG_DEBUG("Skipping %s implementation", (*iter)->get_name_c());
             continue;
         }
 
+        // already checked parent implementation
+        if ((*iter) == parent->m_impl) {
+            LOG_DEBUG("Already checked %s implementation",
+                      (*iter)->get_name_c());
+            continue;
+        }
+
         LOG_DEBUG("Checking if %s is native through implementation %s",
                   name.c_str(), (*iter)->get_name_c());
 
@@ -280,21 +294,18 @@ static GpiObjHdl *__gpi_get_handle_by_name(GpiObjHdl *parent, std::string name,
            be seen discovered even if the parents implementation is not the same
            as the one that we are querying through */
 
-        // std::string &to_query = base->is_this_impl(*iter) ? s_name : fq_name;
-        if ((hdl = (*iter)->native_check_create(name, parent))) {
+        auto hdl = (*iter)->native_check_create(name, parent);
+        if (hdl) {
             LOG_DEBUG("Found %s via %s", name.c_str(), (*iter)->get_name_c());
-            break;
+            return CHECK_AND_STORE(hdl);
         }
     }
 
-    if (hdl)
-        return CHECK_AND_STORE(hdl);
-    else
-        return hdl;
+    return NULL;
 }
 
-static GpiObjHdl *__gpi_get_handle_by_raw(GpiObjHdl *parent, void *raw_hdl,
-                                          GpiImplInterface *skip_impl) {
+static GpiObjHdl *gpi_get_handle_by_raw(GpiObjHdl *parent, void *raw_hdl,
+                                        GpiImplInterface *skip_impl) {
     vector<GpiImplInterface *>::iterator iter;
 
     GpiObjHdl *hdl = NULL;
@@ -325,7 +336,7 @@ static GpiObjHdl *__gpi_get_handle_by_raw(GpiObjHdl *parent, void *raw_hdl,
 
 gpi_sim_hdl gpi_get_handle_by_name(gpi_sim_hdl base, const char *name) {
     std::string s_name = name;
-    GpiObjHdl *hdl = __gpi_get_handle_by_name(base, s_name, NULL);
+    GpiObjHdl *hdl = gpi_get_handle_by_name_(base, s_name, NULL);
     if (!hdl) {
         LOG_DEBUG(
             "Failed to find a handle named %s via any registered "
@@ -388,7 +399,7 @@ gpi_sim_hdl gpi_next(gpi_iterator_hdl iter) {
                 LOG_DEBUG(
                     "Found a name but unable to create via native "
                     "implementation, trying others");
-                next = __gpi_get_handle_by_name(parent, name, iter->m_impl);
+                next = gpi_get_handle_by_name_(parent, name, iter->m_impl);
                 if (next) {
                     return next;
                 }
@@ -400,7 +411,7 @@ gpi_sim_hdl gpi_next(gpi_iterator_hdl iter) {
                 LOG_DEBUG(
                     "Found an object but not accessible via %s, trying others",
                     iter->m_impl->get_name_c());
-                next = __gpi_get_handle_by_raw(parent, raw_hdl, iter->m_impl);
+                next = gpi_get_handle_by_raw(parent, raw_hdl, iter->m_impl);
                 if (next) {
                     return next;
                 }
@@ -580,3 +591,12 @@ void gpi_deregister_callback(gpi_cb_hdl cb_hdl) {
 const char *GpiImplInterface::get_name_c() { return m_name.c_str(); }
 
 const string &GpiImplInterface::get_name_s() { return m_name; }
+
+void gpi_to_user() { LOG_TRACE("Passing control to GPI user"); }
+
+void gpi_to_simulator() {
+    if (sim_ending) {
+        gpi_cleanup();
+    }
+    LOG_TRACE("Returning control to simulator");
+}
