@@ -966,15 +966,21 @@ int VhpiImpl::deregister_callback(GpiCbHdl *gpi_hdl) {
 }
 
 void VhpiImpl::sim_end() {
-    sim_finish_cb->set_call_state(GPI_DELETE);
-    vhpi_control(vhpiFinish, vhpiDiagTimeLoc);
-    check_vhpi_error();
+    // Some sims do not seem to be able to deregister the end of sim callback
+    // so we need to make sure we have tracked this and not call the handler.
+    if (sim_finish_cb->get_call_state() != GPI_DELETE) {
+        sim_finish_cb->set_call_state(GPI_DELETE);
+        vhpi_control(vhpiFinish, vhpiDiagTimeLoc);
+        check_vhpi_error();
+    }
 }
 
 extern "C" {
 
 // Main entry point for callbacks from simulator
 void handle_vhpi_callback(const vhpiCbDataT *cb_data) {
+    gpi_to_user();
+
     VhpiCbHdl *cb_hdl = (VhpiCbHdl *)cb_data->user_data;
 
     if (!cb_hdl) {
@@ -998,7 +1004,7 @@ void handle_vhpi_callback(const vhpiCbDataT *cb_data) {
             }
     }
 
-    return;
+    gpi_to_simulator();
 };
 
 static void register_initial_callback() {
@@ -1011,14 +1017,14 @@ static void register_final_callback() {
     sim_finish_cb->arm_callback();
 }
 
-static void register_embed() {
+static void register_impl() {
     vhpi_table = new VhpiImpl("VHPI");
     gpi_register_impl(vhpi_table);
 }
 
 // pre-defined VHPI registration table
 COCOTBVHPI_EXPORT void (*vhpi_startup_routines[])() = {
-    register_embed, gpi_load_extra_libs, register_initial_callback,
+    register_impl, gpi_entry_point, register_initial_callback,
     register_final_callback, nullptr};
 
 // For non-VHPI compliant applications that cannot find vhpi_startup_routines
@@ -1033,4 +1039,4 @@ COCOTBVHPI_EXPORT void vhpi_startup_routines_bootstrap() {
 }
 }
 
-GPI_ENTRY_POINT(cocotbvhpi, register_embed)
+GPI_ENTRY_POINT(cocotbvhpi, register_impl)
