@@ -27,8 +27,6 @@
 
 #include "VpiImpl.h"
 
-#include <cocotb_utils.h>  // COCOTB_UNUSED
-
 #include <cstring>
 
 extern "C" {
@@ -629,6 +627,8 @@ extern "C" {
 
 // Main re-entry point for callbacks from simulator
 int32_t handle_vpi_callback(p_cb_data cb_data) {
+    gpi_to_user();
+
     int rv = 0;
 
     VpiCbHdl *cb_hdl = (VpiCbHdl *)cb_data->user_data;
@@ -660,10 +660,12 @@ int32_t handle_vpi_callback(p_cb_data cb_data) {
         }
     }
 
+    gpi_to_simulator();
+
     return rv;
 }
 
-static void register_embed() {
+static void register_impl() {
     vpi_table = new VpiImpl("VPI");
     gpi_register_impl(vpi_table);
 }
@@ -679,7 +681,7 @@ static void register_final_callback() {
 }
 
 COCOTBVPI_EXPORT void (*vlog_startup_routines[])() = {
-    register_embed, gpi_load_extra_libs, register_initial_callback,
+    register_impl, gpi_entry_point, register_initial_callback,
     register_final_callback, nullptr};
 
 // For non-VPI compliant applications that cannot find vlog_startup_routines
@@ -693,4 +695,4 @@ COCOTBVPI_EXPORT void vlog_startup_routines_bootstrap() {
 }
 }
 
-GPI_ENTRY_POINT(cocotbvpi, register_embed)
+GPI_ENTRY_POINT(cocotbvpi, register_impl)
